/////////////////////////////////////////////////////////////////////////////
// Name:        device_24LC02.cpp
// Purpose:     serial EEPROM 24LC02
// Package:     ASIX FORTE programmer demo software
//              distributed as-is, without any warranties
// Author:      http://www.asix.net/
// Created:     2016
/////////////////////////////////////////////////////////////////////////////

// ============================================================================
// declarations
// ============================================================================

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all "standard" wxWidgets headers)
#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

// standard header file
#include "device_24LC02.h"

// ============================================================================
// implementation
// ============================================================================

// ----------------------------------------------------------------------------
// class Device24LC02
// ----------------------------------------------------------------------------
// constructor
Device24LC02::Device24LC02()  
{
    //
}

// virtual function override

wxString Device24LC02::GetName()
{
    return _T("24LC02");
}

wxString Device24LC02::GetShortHint()
{
    return _T("I2C serial EEPROM 2048 bits (x8)");
}

wxString Device24LC02::GetLongHint()
{
    wxString s;
    s += "Device pinout (PDIP/SOIC/TSSOP/MSOP package):\n";
    s += "1 ... A0\n";
    s += "2 ... A1\n";
    s += "3 ... A2\n";
    s += "4 ... VSS\n";
    s += "5 ... SDA\n";
    s += "6 ... SCL\n";
    s += "7 ... WP\n";
    s += "8 ... VCC\n";
    s += "\n";
    s += "Connection to FORTE:\n";
    s += "SDA    <-> D\n";
    s += "SCL    <-> C\n";
    s += "VSS    <-> VSS\n";
    s += "VDD    <-> VDD\n";
    return s;
}

int Device24LC02::GetMemoryWordSize(int memory_index)
{
    return 1;
}

int Device24LC02::GetMemoryWordCount(int memory_index)
{
    return 256;
}

void Device24LC02::RegisterActions()
{
    AddAction(_T("Write"), _T("&Write\tF5"), (UserHandler)&Device24LC02::Write);
    AddAction(_T("Read"), _T("&Read\tF6"), (UserHandler)&Device24LC02::Read);
    AddAction(_T("Verify"), _T("&Verify\tF7"), (UserHandler)&Device24LC02::Verify);
    SetActionLayout(3);
}

void Device24LC02::RegisterSettings()
{
    prop_supply = new wxFloatProperty("Supply Voltage", "SupplyVoltage", 3.3);
    AddSetting(prop_supply);
    prop_verify_write = new wxBoolProperty("Verify After Write", "VerifyAfterWrite", true);
    AddSetting(prop_verify_write);
}

// user-defined functions

void Device24LC02::Read()
{
    ReadToBuffer(true);
    RefreshScreen();
}

void Device24LC02::Write()
{
    int i, page;
    unsigned char databyte;
    if(!OpenProg()) {
        AddLog("Cannot open programmer.");
        return;
    }
    MemoryWindowsToScreenBuffer();
    // set supply voltage
    double supply = prop_supply->GetValue();
    prog->QPoweronVdd(20000, (int)(supply*1000));
    ReadStatus("PoweronVdd");
    // 100kHz
    prog->QI2CSetSpeed(I2C_CLK_100kHz);
    ReadStatus("I2CSetSpeed");
    // disable answers
    prog->QSetGPIOAnswer(0);
    ReadStatus("SetGPIOAnswer");
    // write by page
    for(page=0;page<32;page++) {
        // set word address
        prog->QI2CStart(1);
        prog->QI2CWriteByte(0xA0); // I2C control byte
        prog->QI2CWriteByte(page << 3); // I2C word address
        // write page
        for(i=0;i<8;i++) {
            GetScreenBuffer(0,page*8+i,&databyte);
            prog->QI2CWriteByte(databyte);
        }
        prog->QI2CStop();
        // 5 ms write time
        prog->QDelay(5000);
    }
    // enable answers
    prog->QSetGPIOAnswer(1);
    ReadStatus("SetGPIOAnswer");
    // close
    CloseProg();
    AddLog("Write done.");
    // verify after write
    bool verify = prop_verify_write->GetValue();
    if(verify) Verify();
}

void Device24LC02::Verify()
{
    if(ReadToBuffer(false)) {
        AddLog("Verification failed.");
        return;
    }
    // compare
    int i;
    bool ok;
    wxString s;
    unsigned char databyte_exp, databyte_read;
    MemoryWindowsToScreenBuffer();
    ok = true;
    for(i=0;i<256;i++) {
        GetScreenBuffer(0,i,&databyte_exp);
        GetHiddenBuffer(0,i,&databyte_read);
        if(databyte_exp != databyte_read) {
            ok = false;
            s.Printf("address 0x%02X: expected 0x%02X, read 0x%02X",i,databyte_exp,databyte_read);
            AddLog(s);
        }
    }
    if(ok) AddLog("Verification OK.");
    else AddLog("Verification failed.");
}

int Device24LC02::ReadToBuffer(bool use_screen_buffer)
{
    int i, count, status, received_bytes;
    int data[256];
    unsigned char databyte;
    if(!OpenProg()) {
        AddLog("Cannot open programmer.");
        return 1;
    }
    // set supply voltage
    double supply = prop_supply->GetValue();
    prog->QPoweronVdd(20000, (int)(supply*1000));
    ReadStatus("PoweronVdd");
    // 100kHz
    prog->QI2CSetSpeed(I2C_CLK_100kHz);
    ReadStatus("I2CSetSpeed");
    // set word address
    prog->QI2CStart(1);
    ReadStatus("I2CStart");
    prog->QI2CWriteByte(0xA0);
    ReadStatus("I2C control byte");
    prog->QI2CWriteByte(0x00);
    ReadStatus("I2C word address");
    // read memory
    count = 256;
    prog->QI2CStart(1);
    ReadStatus("I2CStart");
    prog->QI2CWriteByte(0xA1);
    ReadStatus("I2C control byte");
    for(i=0;i<(count-1);i++) prog->QI2CReadByte(1);
    prog->QI2CReadByte(0);
    prog->QI2CStop();
    // get data
    while(1) {
        status = prog->AGetStatus(&received_bytes);
        if(!status) {
            count = 0;
            break;
        }
        if(received_bytes >= count) {
            prog->AGetBlock(data, count, &received_bytes);
            break;
        }
    }
    // take 8-bit data and print
    for(i=0;i<count;i++) {
        databyte = data[i] & 0xFF;
        if(use_screen_buffer) SetScreenBuffer(0,i,&databyte);
        else SetHiddenBuffer(0,i,&databyte);
    }
    ReadStatus("I2CStop");
    // close
    CloseProg();
    AddLog("Read done.");
    return 0;
}
