/////////////////////////////////////////////////////////////////////////////
// Name:        device_93LC46B.cpp
// Purpose:     serial EEPROM 93LC46B
// Package:     ASIX FORTE programmer demo software
//              distributed as-is, without any warranties
// Author:      http://www.asix.net/
// Created:     2016
/////////////////////////////////////////////////////////////////////////////

// ============================================================================
// declarations
// ============================================================================

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all "standard" wxWidgets headers)
#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

// standard header file
#include "device_93LC46B.h"

// ============================================================================
// implementation
// ============================================================================

// ----------------------------------------------------------------------------
// class Device93LC46B
// ----------------------------------------------------------------------------
// constructor
Device93LC46B::Device93LC46B()  
{
    //
}

// virtual function override

wxString Device93LC46B::GetName()
{
    return _T("93LC46B");
}

wxString Device93LC46B::GetShortHint()
{
    return _T("1kbit Microwire Compatible Serial EEPROM (x16)");
}

wxString Device93LC46B::GetLongHint()
{
    wxString s;
    s += "Device pinout (PDIP/SOIC/TSSOP/MSOP package):\n";
    s += "1 ... CS\n";
    s += "2 ... CLK\n";
    s += "3 ... DI\n";
    s += "4 ... DO\n";
    s += "5 ... VSS\n";
    s += "6 ... NC\n";
    s += "7 ... NC\n";
    s += "8 ... VCC\n";
    s += "\n";
    s += "Connection to FORTE:\n";
    s += "CS     <-> P\n";
    s += "VCC    <-> VDD\n";
    s += "VSS    <-> GND\n";
    s += "DI     <-> D\n";
    s += "CLK    <-> C\n";
    s += "DO     <-> I\n";
    return s;
}

int Device93LC46B::GetMemoryWordSize(int memory_index)
{
    return 2;
}

int Device93LC46B::GetMemoryWordCount(int memory_index)
{
    return 64;
}

void Device93LC46B::RegisterActions()
{
    AddAction(_T("Write"), _T("&Write\tF5"), (UserHandler)&Device93LC46B::Write);
    AddAction(_T("Read"), _T("&Read\tF6"), (UserHandler)&Device93LC46B::Read);
    AddAction(_T("Verify"), _T("&Verify\tF7"), (UserHandler)&Device93LC46B::Verify);
    SetActionLayout(3);
}

void Device93LC46B::RegisterSettings()
{
    prop_supply_rd = new wxFloatProperty("Supply Voltage Read", "SupplyVoltageRead", 3.3);
    AddSetting(prop_supply_rd);
    prop_supply_wr = new wxFloatProperty("Supply Voltage Write", "SupplyVoltageWrite", 3.3);
    AddSetting(prop_supply_wr);
    prop_verify_write = new wxBoolProperty("Verify After Write", "VerifyAfterWrite", true);
    AddSetting(prop_verify_write);
}

// user-defined functions

void Device93LC46B::Read()
{
    ReadToBuffer(true);
    RefreshScreen();
}

void Device93LC46B::Write()
{
    int i;
    unsigned char databyte[2];
    if(!OpenProg()) {
        AddLog("Cannot open programmer.");
        return;
    }
    MemoryWindowsToScreenBuffer(); // update content
    // initiate
    prog->QSetGPIOAnswer(0);
    ReadStatus("SetGPIOAnswer");

    // set supply voltage
    double supply = prop_supply_wr->GetValue();
    prog->QPoweronVdd(20000, (int)(supply*1000));
    ReadStatus("PoweronVdd");

    prog->QSetShiftSpeed(SHIFT_CLK_1000kHz);
    prog->QSetPins((PINS_LO << PINS_P_BIT) | (PINS_LO << PINS_C_BIT) | (PINS_LO << PINS_D_BIT)); // CS=0, CLK=0, DI=0
    prog->QDelay(1);
    
    // EWEN command
    prog->QSetPins((PINS_HI << PINS_P_BIT)); // CS=1
    prog->QDelay(1);
    prog->QShiftBits(0x019, SHIFT_MODE0, 9);
    prog->QDelay(1);
    prog->QSetPins((PINS_LO << PINS_P_BIT)); // CS=0
    prog->QDelay(1);

    for(i=0;i<64;i++) {
        // WRITE command
        prog->QSetPins((PINS_HI << PINS_P_BIT)); // CS=1
        prog->QDelay(1);
        prog->QShiftBits(0x005 | (((int)LsbToMsb(i)) << 1), SHIFT_MODE0, 9);
        // data
        GetScreenBuffer(0,i,databyte);
        prog->QShiftBits(LsbToMsb(databyte[1]), SHIFT_MODE0, 8);
        prog->QShiftBits(LsbToMsb(databyte[0]), SHIFT_MODE0, 8);
        prog->QDelay(1);
        prog->QSetPins((PINS_LO << PINS_P_BIT)); // CS=0
        prog->QDelay(8000); // wtite time
    }
    // enable answers
    prog->QSetGPIOAnswer(1);
    ReadStatus("SetGPIOAnswer");
    // close
    CloseProg();
    AddLog("Write done.");
    // verify after write
    bool verify = prop_verify_write->GetValue();
    if(verify) Verify();
}

void Device93LC46B::Verify()
{
    if(ReadToBuffer(false)) {
        AddLog("Verification failed.");
        return;
    }
    // compare
    int i;
    bool ok;
    wxString s;
    unsigned char databyte_exp[2], databyte_read[2];
    MemoryWindowsToScreenBuffer();
    ok = true;
    for(i=0;i<64;i++) {
        GetScreenBuffer(0,i,databyte_exp);
        GetHiddenBuffer(0,i,databyte_read);
        if((databyte_exp[0] != databyte_read[0]) || (databyte_exp[1] != databyte_read[1])) {
            ok = false;
            s.Printf("address 0x%02X: expected 0x%02X%02X, read 0x%02X%02X",i,databyte_exp[1],databyte_exp[0],databyte_read[1],databyte_read[0]);
            AddLog(s);
        }
    }
    if(ok) AddLog("Verification OK.");
    else AddLog("Verification failed.");
}

int Device93LC46B::ReadToBuffer(bool use_screen_buffer)
{
    int i, count, status, received_bytes;
    int data[128];
    unsigned char databyte[2];
    if(!OpenProg()) {
        AddLog("Cannot open programmer.");
        return 1;
    }
    // initiate
    prog->QSetGPIOAnswer(0);
    ReadStatus("SetGPIOAnswer");

    // set supply voltage
    double supply = prop_supply_rd->GetValue();
    prog->QPoweronVdd(20000, (int)(supply*1000));
    ReadStatus("PoweronVdd");

    prog->QSetShiftSpeed(SHIFT_CLK_1000kHz);
    prog->QSetPins((PINS_LO << PINS_P_BIT) | (PINS_LO << PINS_C_BIT) | (PINS_LO << PINS_D_BIT)); // CS=0, CLK=0, DI=0
    prog->QDelay(1);
    prog->QSetPins((PINS_HI << PINS_P_BIT)); // CS=1
    prog->QDelay(1);
    
    // 9-bit command
    prog->QShiftBits(0x003, SHIFT_MODE0, 9);
    // read memory
    count = 128; // bytes
    prog->QShiftBytes_In(SHIFT_MODE1, SHIFT_OUTIN_PINI, count);
    prog->QSetPins((PINS_LO << PINS_P_BIT)); // CS=0
    // get data
    while(1) {
        status = prog->AGetStatus(&received_bytes);
        if(!status) {
            count = 0;
            break;
        }
        if(received_bytes >= count) {
            prog->AGetBlock(data, count, &received_bytes);
            break;
        }
    }
    
    // take 8-bit data and save to 16-bit buffer
    for(i=0;i<(count/2);i++) {
        databyte[1] = LsbToMsb(data[i*2] & 0xFF);
        databyte[0] = LsbToMsb(data[i*2+1] & 0xFF);
        if(use_screen_buffer) SetScreenBuffer(0,i,databyte);
        else SetHiddenBuffer(0,i,databyte);
    }
    
    prog->QSetGPIOAnswer(1);
    ReadStatus("SetGPIOAnswer");
    
    // close
    CloseProg();
    AddLog("Read done.");
    return 0;
}

unsigned char Device93LC46B::LsbToMsb(unsigned char databyte)
{
    int i;
    unsigned char x = 0;
    for(i=0;i<8;i++) {
        x = x | (((databyte >> i) & 1) << (7-i));
    }
    return x;
}


