/////////////////////////////////////////////////////////////////////////////
// Name:        device_M25P80.cpp
// Purpose:     serial EEPROM M25P80
// Package:     ASIX FORTE programmer demo software
//              distributed as-is, without any warranties
// Author:      http://www.asix.net/
// Created:     2016
/////////////////////////////////////////////////////////////////////////////

// ============================================================================
// declarations
// ============================================================================

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all "standard" wxWidgets headers)
#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

// standard header file
#include "device_M25P80.h"

// ============================================================================
// implementation
// ============================================================================

// ----------------------------------------------------------------------------
// class DeviceM25P80
// ----------------------------------------------------------------------------
// constructor
DeviceM25P80::DeviceM25P80()  
{
    //
}

// virtual function override

wxString DeviceM25P80::GetName()
{
    return _T("M25P80");
}

wxString DeviceM25P80::GetShortHint()
{
    return _T("SPI serial FLASH 8 Mbit");
}

wxString DeviceM25P80::GetLongHint()
{
    wxString s;
    s += "Device pinout (SO8/VFQFPN package):\n";
    s += "1 ... S#\n";
    s += "2 ... DQ1\n";
    s += "3 ... W#\n";
    s += "4 ... VSS\n";
    s += "5 ... DQ0\n";
    s += "6 ... C\n";
    s += "7 ... HOLD#\n";
    s += "8 ... VCC\n";
    s += "\n";
    s += "Connection to FORTE:\n";
    s += "S#     <-> P\n";
    s += "VCC    <-> VDD\n";
    s += "VSS    <-> GND\n";
    s += "DQ0    <-> D\n";
    s += "C      <-> C\n";
    s += "DQ1    <-> I\n";
    s += "W#     <-> VDD\n";
    s += "HOLD#  <-> VDD\n";
    return s;
}

int DeviceM25P80::GetMemoryWordSize(int memory_index)
{
    return 1;
}

int DeviceM25P80::GetMemoryWordCount(int memory_index)
{
    return 1024*1024;
}

void DeviceM25P80::RegisterBitAliases()
{
    AddBitAlias("first byte",0,0,0,8);
    AddBitAlias("second byte",0,1,0,8);
    AddBitAlias("bit 7 of third byte",0,2,7,1);
    AddBitAlias("last byte",0,GetMemoryWordCount(0)-1,0,8);
}

void DeviceM25P80::RegisterActions()
{
    AddAction(_T("Read"), _T("&Read\tF6"), (UserHandler)&DeviceM25P80::Read);
    SetActionLayout(1);
}

// user-defined functions

void DeviceM25P80::Read()
{
    if(!OpenProg()) {
        AddLog("Cannot open programmer.");
        return;
    }
    //
    int i, n, mem_bytes_count, status, received_bytes;
    int data_count, buffer_count, buffer_size, block_in_size;
    int *data, *buffer;
    wxString s;
    unsigned char databyte;
    // initiate
    prog->QSetGPIOAnswer(0);
    ReadStatus("SetGPIOAnswer");
    prog->QSetActiveLED(LED_ACT_Y_FAST_BLINK);
    ReadStatus("SetActiveLED");
    prog->QPoweronVdd(20000, 3300); // supply voltage 3.3V
    ReadStatus("PoweronVdd");
    prog->QSetShiftSpeed(SHIFT_CLK_10000kHz);
    prog->QSetPins((PINS_HI << PINS_P_BIT) | (PINS_LO << PINS_C_BIT) | (PINS_LO << PINS_D_BIT)); // S#=1, C=0, DQ0=0
    prog->QDelay(100);
    prog->QSetPins((PINS_LO << PINS_P_BIT)); // S#=0
    prog->QDelay(1);
    // command code and address
    prog->QShiftByte(0xC0, SHIFT_MODE0);
    prog->QShiftByte(0x00, SHIFT_MODE0);
    prog->QShiftByte(0x00, SHIFT_MODE0);
    prog->QShiftByte(0x00, SHIFT_MODE0);
    // read memory
    mem_bytes_count = 1024*1024; // total read bytes
    block_in_size = 512; // bytes in one QShiftBytes_In()
    buffer_size = 16384; // bytes in one AGetBlock()
    s.Printf("Reading %d bytes ...", mem_bytes_count);
    AddLog(s);
    data = new int[mem_bytes_count];
    data_count = 0;
    buffer = new int[buffer_size];
    while(data_count < mem_bytes_count) {
        buffer_count = 0;
        while((buffer_count < buffer_size) && (data_count+buffer_count < mem_bytes_count)) {
            n = mem_bytes_count - data_count - buffer_count;
            if(n > block_in_size) n = block_in_size;
            prog->QShiftBytes_In(SHIFT_MODE0, SHIFT_OUTIN_PINI, n);
            buffer_count += n;
        }
        while(1) {
            status = prog->AGetStatus(&received_bytes);
            if(!status) break;
            if(received_bytes >= buffer_count) break;
        }
        if(!status) break;
        prog->AGetBlock(buffer, buffer_count, &received_bytes);
        for(i=0;i<buffer_count;i++) data[data_count+i] = buffer[i];
        data_count += buffer_count;
    }
    //
    prog->QSetPins((PINS_HI << PINS_P_BIT)); // S#=1
    prog->QSetActiveLED(LED_ACT_OFF);
    ReadStatus("SetActiveLED");
    // take 8-bit data and print
    if(status) {
        for(i=0;i<mem_bytes_count;i++) {
            databyte = LsbToMsb(data[i] & 0xFF);
            SetScreenBuffer(0,i,&databyte);
        }
        RefreshScreen();
    }
    else {
        AddLog("Data read failed");
    }
    //
    prog->QSetGPIOAnswer(1);
    ReadStatus("SetGPIOAnswer");
    delete data;
    delete buffer;
    CloseProg();
    AddLog("Read done.");
}

unsigned char DeviceM25P80::LsbToMsb(unsigned char databyte)
{
    int i;
    unsigned char x = 0;
    for(i=0;i<8;i++) {
        x = x | (((databyte >> i) & 1) << (7-i));
    }
    return x;
}
